package websockets;

import io.undertow.Undertow;
import io.undertow.websockets.WebSocketConnectionCallback;
import io.undertow.websockets.core.WebSocketChannel;
import io.undertow.websockets.spi.WebSocketHttpExchange;
import org.xnio.ChannelListener;
import tcp.CuisineTCPServer;
import websockets.handlers.CuisineWebSocketReceiveHandler;
import websockets.handlers.CuisineWebSocketWebSocketChichatHandler;
import websockets.handlers.CuisineWebSocketWebSocketTemperatureHandler;

import static io.undertow.Handlers.path;
import static io.undertow.Handlers.websocket;

public class CuisineWebSocketServer {

    CuisineWebSocketConnectionHandler fConnectionHandler;
    CuisineWebSocketConnectionHandler fChatConnectionHandler;
    CuisineWebSocketWebSocketTemperatureHandler fMessageHandler;
    CuisineWebSocketWebSocketChichatHandler fChitchatMessageHandler;
    Undertow fWebSocketServer;
    String fWebSocketHost;
    int fWebSocketPort;
    int fConnectedClients;

    public CuisineWebSocketServer(String iHost, int iPort
    ) {
        fWebSocketPort = iPort;
        fWebSocketHost = iHost;
    }

    public void initTCPServer(CuisineTCPServer iTcpServer) {
        fMessageHandler = new CuisineWebSocketWebSocketTemperatureHandler(iTcpServer);
    }

    public void start() {
        fChitchatMessageHandler = new CuisineWebSocketWebSocketChichatHandler();

        fConnectionHandler = new CuisineWebSocketConnectionHandler(fMessageHandler);
        fChatConnectionHandler = new CuisineWebSocketConnectionHandler(fChitchatMessageHandler);
        System.out.println("Opening web socket server at "+fWebSocketHost+":"+fWebSocketPort);
        fWebSocketServer = Undertow.builder()
                .addHttpListener(fWebSocketPort, "0.0.0.0")
                .setHandler(
                        path()
                                .addPrefixPath("/data", websocket(fConnectionHandler))
                                .addPrefixPath("/chat", websocket(fChatConnectionHandler))
                )
                .build();
        fWebSocketServer.start();
    }


    private class CuisineWebSocketConnectionHandler implements WebSocketConnectionCallback {
        private CuisineWebSocketReceiveHandler fWebSocketMessageHandler;

        protected CuisineWebSocketConnectionHandler(CuisineWebSocketReceiveHandler iWebSocketMessageHandler) {
            fWebSocketMessageHandler = iWebSocketMessageHandler;
        }

        @Override
        public void onConnect(WebSocketHttpExchange exchange, WebSocketChannel channel) {
            channel.getReceiveSetter().set(fWebSocketMessageHandler);
            fWebSocketMessageHandler.register(channel);
            channel.addCloseTask(new ChannelListener<WebSocketChannel>(){

                @Override
                public void handleEvent(WebSocketChannel channel) {
                    fWebSocketMessageHandler.unregister(channel);
                    System.out.println(channel.getPeerAddress());
                    System.out.println("Connection closed");
                }
            });
            channel.resumeReceives();
        }

    }


}
